package sun.awt.image;

import sun.java2d.SurfaceDataProxy;
import java.awt.GraphicsEnvironment;
import java.awt.image.VolatileImage;
import java.util.Iterator;
import java.awt.ImageCapabilities;
import java.awt.GraphicsConfiguration;
import sun.java2d.SurfaceData;
import java.awt.image.BufferedImage;
import java.awt.Image;
import java.util.concurrent.ConcurrentHashMap;

public abstract class SurfaceManager
{
    private static SurfaceManager.ImageAccessor imgaccessor;
    private ConcurrentHashMap<Object, Object> cacheMap;
    
    public SurfaceManager() {
    }
    
    public static void setImageAccessor(SurfaceManager.ImageAccessor imgaccessor) {
        if (SurfaceManager.imgaccessor != null) {
            throw new InternalError("Attempt to set ImageAccessor twice");
        }
        SurfaceManager.imgaccessor = imgaccessor;
        return;
    }
    
    public static SurfaceManager getManager(Image image) {
        SurfaceManager surfaceManager = SurfaceManager.imgaccessor.getSurfaceManager(image);
        if (surfaceManager == null) {
            try {
                BufferedImage bufferedImage = (BufferedImage)image;
                surfaceManager = new BufImgSurfaceManager(bufferedImage);
                setManager(bufferedImage, surfaceManager);
            }
            catch (ClassCastException ex) {
                throw new IllegalArgumentException("Invalid Image variant");
            }
        }
        return surfaceManager;
    }
    
    public static void setManager(Image image, SurfaceManager surfaceManager) {
        SurfaceManager.imgaccessor.setSurfaceManager(image, surfaceManager);
    }
    
    public Object getCacheData(Object key) {
        return (this.cacheMap == null) ? null : this.cacheMap.get(key);
    }
    
    public void setCacheData(Object key, Object value) {
        if (this.cacheMap == null) {
            synchronized (this) {
                if (this.cacheMap == null) {
                    this.cacheMap = new ConcurrentHashMap(2);
                }
            }
        }
        this.cacheMap.put(key, value);
    }
    
    public abstract SurfaceData getPrimarySurfaceData();
    
    public abstract SurfaceData restoreContents();
    
    public void acceleratedSurfaceLost() {
    }
    
    public ImageCapabilities getCapabilities(GraphicsConfiguration graphicsConfiguration) {
        return new SurfaceManager.ImageCapabilitiesGc(graphicsConfiguration);
    }
    
    public synchronized void flush() {
        this.flush(false);
    }
    
    synchronized void flush(boolean b) {
        if (this.cacheMap != null) {
            Iterator<Object> iterator = this.cacheMap.values().iterator();
            while (iterator.hasNext()) {
                Object next = iterator.next();
                if (next instanceof SurfaceManager.FlushableCacheData && ((SurfaceManager.FlushableCacheData)next).flush(b)) {
                    iterator.remove();
                }
                else {
                    continue;
                }
            }
        }
    }
    
    public void setAccelerationPriority(float n) {
        if (n == 0.0f) {
            this.flush(true);
        }
    }
    
    public static int getImageScale(Image image) {
        if (!(image instanceof VolatileImage)) {
            return 1;
        }
        return getManager(image).getPrimarySurfaceData().getDefaultScale();
    }
    
    public abstract static class ImageAccessor
    {
        public abstract SurfaceManager getSurfaceManager(Image p0);
        
        public abstract void setSurfaceManager(Image p0, SurfaceManager p1);
    }
    
    class ImageCapabilitiesGc extends ImageCapabilities
    {
        GraphicsConfiguration gc;
        
        public ImageCapabilitiesGc(GraphicsConfiguration gc) {
            super(false);
            this.gc = gc;
        }
        
        @Override
        public boolean isAccelerated() {
            GraphicsConfiguration graphicsConfiguration = this.gc;
            if (graphicsConfiguration == null) {
                graphicsConfiguration = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
            }
            if ((graphicsConfiguration instanceof SurfaceManager.ProxiedGraphicsConfig)) {
                Object proxyKey = ((SurfaceManager.ProxiedGraphicsConfig)graphicsConfiguration).getProxyKey();
                if (proxyKey != null) {
                    SurfaceDataProxy surfaceDataProxy = (SurfaceDataProxy)SurfaceManager.this.getCacheData(proxyKey);
                    return surfaceDataProxy != null && surfaceDataProxy.isAccelerated();
                }
            }
            return false;
        }
    }
    
    public interface FlushableCacheData
    {
        public abstract boolean flush(boolean p0);
    }
    
    public interface ProxiedGraphicsConfig
    {
        public abstract Object getProxyKey();
    }
}
